/*
 * Copyright(C) OASIS(R) 2005,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */
package org.oasisopen.sca;

import java.util.Collection;

/**
 * The following Java code defines the ComponentContext interface.
 */
public interface ComponentContext {

    /**
     * Returns the absolute URI of the component within the SCA domain.
     * 
     * @return the absolute URI of the component within the SCA domain
     */
    String getURI();

    /**
     * Returns a proxy for the reference defined by the current component.
     *
     * The getService() method takes as its input arguments the Java type
     * used to represent the target service on the client and the name of
     * the service reference. It returns an object providing access to
     * the service. The returned object implements the Java interface the
     * service is typed with.
     *
     * This method MUST throw an IllegalArgumentException if the reference
     * has multiplicity greater than one.
     * 
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @param     referenceName the name of the service reference
     * @return    a proxy for the reference defined by the current component
     * @exception IllegalArgumentException if the reference has multiplicity
     *            greater than one
     */
    <B> B getService(Class<B> businessInterface, String referenceName)
    		throws IllegalArgumentException;

    /**
     * Returns a ServiceReference defined by the current component.
     *
     * This method MUST throw an IllegalArgumentException if the reference
     * has multiplicity greater than one.
     * 
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @param     referenceName the name of the service reference
     * @return    a ServiceReference defined by the current component
     * @exception IllegalArgumentException if the reference has multiplicity
     *            greater than one
     */
    <B> ServiceReference<B> getServiceReference(Class<B> businessInterface, 
                                                String referenceName)
            throws IllegalArgumentException;

    /**
     * Returns a list of typed service proxies for a business interface type
     * and a reference name.
     *
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @param     referenceName the name of the service reference
     * @return    a list of typed service proxies for a business interface
     *            type and a reference name
     */
    <B> Collection<B> getServices(Class<B> businessInterface,
                                  String referenceName)
     		throws IllegalArgumentException; 

    /**
     * Returns a list typed service references for a business interface type
     * and a reference name.
     *
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @param     referenceName the name of the service reference
     * @return    a list typed service references for a business
     *            interface type and a reference name
     */
    <B> Collection<ServiceReference<B>> getServiceReferences(
                       Class<B> businessInterface, String referenceName)
     		throws IllegalArgumentException; 

    /**
     * Returns a ServiceReference that can be used to invoke this component
     * over the designated service.
     *
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @return    a ServiceReference that can be used to invoke this
     *            component over the designated service
     */
    <B> ServiceReference<B> createSelfReference(Class<B> businessInterface)
     		throws IllegalArgumentException;

    /**
     * Returns a ServiceReference that can be used to invoke this component
     * over the designated service. Service name explicitly declares the
     * service name to invoke.
     *
     * @param     <B> the Java type used to represent the target service
     *            on the client
     * @param     businessInterface the Class object for the Java type used
     *            to represent the target service on the client
     * @param     serviceName the service name to invoke
     * @return    a ServiceReference that can be used to invoke this
     *            component over the designated service
     */
    <B> ServiceReference<B> createSelfReference(Class<B> businessInterface, 
                                                String serviceName)
     		throws IllegalArgumentException;

    /**
     * Returns the value of an SCA property defined by this component.
     *
     * @param     <B> the property type
     * @param     type the Class object for the property type
     * @param     propertyName the property name
     * @return    The value of an SCA property defined by this component
     */
    <B> B getProperty(Class<B> type, String propertyName)
		    throws IllegalArgumentException;

    /**
     * Casts a type-safe reference to a ServiceReference.
     *
     * @param     <B> the type of the type-safe reference
     * @param     <R> the return type
     * @param     target the type-safe reference
     * @return    a ServiceReference
     * @exception IllegalArgumentException
     */
    <B> ServiceReference<B> cast(B target) 
            throws IllegalArgumentException;

    /**
     * Returns the context for the current SCA service request, or null
     * if there is no current request or if the context is unavailable.
     *
     * This method MUST return non-null when invoked during the execution
     * of a Java business method for a service operation or callback
     * operation, on the same thread that the SCA runtime provided,
     * and MUST return null in all other cases.
     *
     * @return    the context for the current SCA service request
     */
    RequestContext getRequestContext();
}
